require "GameLib"


local CombatMusicVolume = {} 

local sMusicConsoleVar = "sound.volumeMusic"
local nTimerUpdateInterval = 0.05
local tDefaultSettings = {
	nInCombatMusicVolume = nil,
	nMusicVolume = nil,
	bFade = true,
	nFadeTime = 2,
}


function CombatMusicVolume:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self 

	self.bFading = false
	self.bInCombat = false
	self.nCurVolume = nil
	self.nFadeVolDiff = 0
	self.bFadeUp = false
	self.nFadeProgress = 0
	self.nFadeSteps = 0
	self.nFadeVolumeStep = 0
	self.nVolLow = 0
	self.nVolHigh = 0

	self.settings = {}

	for k, v in pairs(tDefaultSettings) do
		self.settings[k] = v
	end

	return o
end

function CombatMusicVolume:Init()
	Apollo.RegisterAddon(self)
end

function CombatMusicVolume:OnLoad()
	Apollo.RegisterSlashCommand("cmv", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("vcmv", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("combatmusicvolume", "OnSlashCommand", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
	self.timerUpdater = ApolloTimer.Create(nTimerUpdateInterval, true, "OnFadeTimer", self)
	self.timerUpdater:Stop()

	self.settings.nMusicVolume = Apollo.GetConsoleVariable(sMusicConsoleVar)
	if self.settings.nInCombatMusicVolume == nil and self.settings.nMusicVolume > 0 then
		self.settings.nInCombatMusicVolume = self.settings.nMusicVolume / 2
	end
end

function CombatMusicVolume:OnEnteredCombat(unit, bInCombat)
	if unit == GameLib.GetPlayerUnit() then
		self.bInCombat = bInCombat
		if bInCombat then
			if not self.bFading then
				self.settings.nMusicVolume = Apollo.GetConsoleVariable(sMusicConsoleVar)
			end
			if self.settings.bFade then
				self:StartFade(true)
			else
				Apollo.SetConsoleVariable(sMusicConsoleVar, self.settings.nInCombatMusicVolume)
			end
		else
			if self.settings.bFade then
				self:StartFade(false)
			else
				Apollo.SetConsoleVariable(sMusicConsoleVar, self.settings.nMusicVolume)
			end
		end
	end
end

function CombatMusicVolume:StartFade(bInCombat)
	if self.settings.nMusicVolume == self.settings.nInCombatMusicVolume then return end
	if self.nCurVolume == nil then
		self.nCurVolume = bInCombat and self.settings.nMusicVolume or self.settings.nInCombatMusicVolume
	end
	if self.settings.nInCombatMusicVolume > self.settings.nMusicVolume then
		self.nVolLow = self.settings.nMusicVolume
		self.nVolHigh = self.settings.nInCombatMusicVolume
	else
		self.nVolLow = self.settings.nInCombatMusicVolume
		self.nVolHigh = self.settings.nMusicVolume
	end
	if self.bFading then
		if self.nCurVolume ~= self.settings.nInCombatMusicVolume and self.nCurVolume ~= self.settings.nMusicVolume then
			self.nFadeProgress = self.nFadeSteps - self.nFadeProgress
		end
	else
		if self.nCurVolume ~= self.settings.nMusicVolume and bInCombat then
			self.nCurVolume = self.settings.nMusicVolume
		end
	end
	self.nFadeVolDiff = bInCombat and self.settings.nMusicVolume - self.settings.nInCombatMusicVolume or self.settings.nInCombatMusicVolume - self.settings.nMusicVolume
	if self.nFadeVolDiff < 0 then
		self.bFadeUp = true
		self.nFadeVolDiff = math.abs(self.nFadeVolDiff)
	else
		self.bFadeUp = false
	end
	self.nFadeSteps = math.floor((self.settings.nFadeTime / nTimerUpdateInterval) + 0.5)
	self.nFadeVolumeStep = self.nFadeVolDiff / self.nFadeSteps
	if not self.bFading then
		self.timerUpdater:Start()
		self.bFading = true
	end
end

function CombatMusicVolume:OnFadeTimer()
	if not self.bFading then return end
	self.nFadeProgress = self.nFadeProgress + 1
	if self.nFadeProgress < self.nFadeSteps then
		self.nCurVolume = self.bFadeUp and self.nCurVolume + self.nFadeVolumeStep or self.nCurVolume - self.nFadeVolumeStep
		if self.nCurVolume >= self.nVolHigh or self.nCurVolume <= self.nVolLow then
			self:StopFadeTimer()
		else
			Apollo.SetConsoleVariable(sMusicConsoleVar, self.nCurVolume)
		end
	else
		self:StopFadeTimer()
	end
end

function CombatMusicVolume:StopFadeTimer()
	self.timerUpdater:Stop()
	self.nCurVolume = self.bInCombat and self.settings.nInCombatMusicVolume or self.settings.nMusicVolume
	Apollo.SetConsoleVariable(sMusicConsoleVar, self.nCurVolume)
	self.bFading = false
	self.nFadeProgress = 0
end

function CombatMusicVolume:OnSlashCommand(strCmd, strParam)
	if strParam ~= "" then
		local tParams = {}
		for w in string.gmatch(strParam, "%S*") do
			if w ~= "" then
				if tonumber(w) then
					table.insert(tParams, tonumber(w))
				else
					table.insert(tParams, string.lower(w))
				end
			end
		end
		if tParams[1] == "music" then
			if tParams[2] and tonumber(tParams[2]) then
				if tParams[2] >= 0 and tParams[2] <= 100 then
					self.settings.nInCombatMusicVolume = (math.floor(tParams[2] + 0.5)) / 100
					self:WriteToChat("Combat music volume set to " .. self.settings.nInCombatMusicVolume * 100 .. "%.")
				else
					self:WriteToChat("Value needs to be a number between 0 and 100.")
				end
			else
				self:WriteToChat("You need to specify a number between 0 and 100.")
			end
		elseif tParams[1] == "fade" then
			self.settings.bFade = not self.settings.bFade
			if self.settings.bFade then
				self:WriteToChat("Now fading volume in and out over " .. self.settings.nFadeTime .. " second(s).")
			else
				self:WriteToChat("Volume fading disabled.")
			end
		elseif tParams[1] == "time" then
			if tParams[2] and tonumber(tParams[2]) then
				if tParams[2] >= 0.1 and tParams[2] <= 5 then
					self.settings.nFadeTime = tParams[2]
					self:WriteToChat("Volume fade time set to " .. self.settings.nFadeTime .. " second(s).")
				else
					self:WriteToChat("Value needs to be a number between 0.1 and 5.")
				end
			else
				self:WriteToChat("You need to specify a number between 0.1 and 5.")
			end
		else
			self:ShowHelp()
		end
	else
		self:ShowHelp()
	end
end

function CombatMusicVolume:ShowHelp()
	local sFadeState = "Off"
	if self.settings.bFade then sFadeState = "On" end
	self:WriteToChat("Available commands for Combat Music Volume:")
	self:WriteToChat("\"music x\" sets the music volume during combat. x needs to be a number between 0 and 100. Currently: " .. math.floor((self.settings.nInCombatMusicVolume * 100) + 0.5) .. "%.")
	self:WriteToChat("\"fade\" toggles volume fading on/off. Currently: " .. sFadeState .. ".")
	self:WriteToChat("\"time x\" sets the fade time (in seconds). x needs to be a number between 0.1 and 4. Currently: " .. self.settings.nFadeTime .. " second(s).")
	self:WriteToChat("So for instance to change the music volume to 60 during combat you would type in \"/vcmv music 60\".")
end

function CombatMusicVolume:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then return nil end
	return self.settings
end

function CombatMusicVolume:OnRestore(eLevel, tData)
	if tData then
		for k, v in pairs(tData) do
			self.settings[k] = v
		end
		self.settings.nMusicVolume = Apollo.GetConsoleVariable(sMusicConsoleVar)
	end
end

function CombatMusicVolume:WriteToChat(text)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, text)
end



local CombatMusicVolumeInst = CombatMusicVolume:new()
CombatMusicVolumeInst:Init()
